#!/usr/bin/perl

use strict;
use warnings;

use POSIX ();

my %CIDRMAP = (
    32  => '',
    24  => '/255.255.255.0',
    16  => '/255.255.0.0',
    8   => '/255.0.0.0',
    0   => '/0.0.0.0',
);

my $basename = $0;
$basename =~ s%.*/%%;
if ((@ARGV < 1) || ($ARGV[0] !~ /^[a-zA-Z_][a-zA-Z0-9)_]*$/)) {
    die "Usage: $basename <shellvarname>";
}

my $list_started = 0;
my $shellvar=$ARGV[0];
my @networks;
my $malformed = 0;

my $line;
OUTERLOOP: while (defined(($line = <STDIN>))) {
    chomp $line;
    if ($line =~ /^Members of /) {
        $list_started = 1;
        next OUTERLOOP;
    } elsif (!$list_started) {
        next OUTERLOOP;
    } elsif ($line =~ /^\s*$/) {
        next OUTERLOOP;
    }

    if ($line !~ /^\s*(\d+)\.(\d+)\.(\d+)\.(\d+)\s*$/) {
        $malformed++;
        next OUTERLOOP;
    }
    my @octets = ($1, $2, $3, $4);
    my $netmask = 32;
    my $had_nonzero = 0;
    foreach my $octet (reverse(@octets)) {
        if (($octet < 0) || ($octet > 255)) {
            $malformed++;
            next OUTERLOOP;
        }
        if (!$had_nonzero && ($octet == 0)) {
            $netmask -= 8;
        } else {
            $had_nonzero = 1;
        }
    }

    my $ip = join('.', map { 0 + $_ } @octets);
    my $dotmask = $CIDRMAP{$netmask};
    push(@networks, "${ip}$dotmask");
}

if (!$list_started) {
    die "Did not find start of IP address list";
}
if ($malformed > 0) {
    warn "Dropped $malformed input line(s)";
}

print '# generated ', POSIX::asctime(localtime());

for (my $ii = 0; $ii < @networks; $ii++) {
    if ($ii == 0) {
        print $shellvar, '="';
    } else {
        print ' ' x (length($shellvar) + 2);
    }

    if ($ii != $#networks) {
        printf("%-32s\\\n", $networks[$ii]);
    } else {
        print $networks[$ii], "\"\n";
    }
}
